#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <help.h>
#include <ctype.h>

#include <parse.h>

int main(int argc, char **argv) {
    FILE *fp;
    FILE *fp_map;
    size_t size;
    char tiles[256];
    int i, a, len, out;
    int w, h, pw, ph, tw, th, padx, pady, dw, dh;
    bool found;
    Tokendata *tokens = NULL;
    char buffer[TOKEN_SZ], c;
    char *data = NULL;
    int amount;
    if(argc < 2){
        puts(help);
        return 1;
    }
    /* Loading config */
    fp = fopen(argv[1], "rb");
    if(fp == NULL){
        printf("[mapconv] Can't open \"%s\"\n", argv[1]);
        return 2;
    }
    fseek(fp, 0L, SEEK_END);
    size = ftell(fp);
    rewind(fp);
    data = malloc(sizeof(unsigned char) * size);
    if(data == NULL){
        puts("[mapconv] More memory needed!");
        return 3;
    }
    if(!fread(data, 1, sizeof(unsigned char) * size, fp)){
        fclose(fp);
        return 4;
    }
    fclose(fp);
    puts("[status] Parsing config ...");
    out = parse(data, &tokens);
    if(out < 0){
        puts("[mapconv] Error when parsing config!");
        return -1;
    }
    puts("[status] Parsing config successful!");
    /* TODO the following lines are really dirty */
    /* Get the width and the height */
    i=get_value_from_name(T_INT, "width", tokens, out);
    w = atoi(tokens[i].content);
    i=get_value_from_name(T_INT, "height", tokens, out);
    h=atoi(tokens[i].content);
    len=0;
    puts("[status] Getting tile numbers.");
    for(i=0;i<256;i++){
        sprintf(buffer, "t%d", i);
        a = get_value_from_name_noerror(T_CHAR, buffer, tokens, out);
        if(a<0) break;
        tiles[i] = tokens[a].content[0];
        len++;
    }
    free(data);
    /* Make the map */
    puts("[status] Reading map.");
    i=get_value_from_name(T_STRING, "input", tokens, out);
    fp = fopen(tokens[i].content, "rb");
    if(fp == NULL){
        printf("[mapconv] Can't open \"%s\"\n", tokens[i].content);
        return 2;
    }
    fseek(fp, 0L, SEEK_END);
    size = ftell(fp);
    rewind(fp);
    data = malloc(sizeof(unsigned char) * size);
    if(data == NULL){
        puts("[mapconv] More memory needed!");
        return 3;
    }
    if(!fread(data, 1, sizeof(unsigned char) * size, fp)){
        fclose(fp);
        return 4;
    }
    fclose(fp);
    i=get_value_from_name(T_STRING, "output", tokens, out);
    fp_map = fopen(tokens[i].content, "w");
    if(fp == NULL){
        printf("[mapconv] Can't open \"%s\"\n", tokens[i].content);
        return 2;
    }
    a=get_value_from_name(T_STRING, "mapname", tokens, out);
    for(i=0;i<(int)strlen(tokens[a].content);i++){
        c = tokens[a].content[i];
        if((c >= 0x30 && c <= 0x39) || (c >= 0x41 && c <= 0x5A) ||
            (c >= 0x61 && c <= 0x7A)){
            buffer[i] = toupper(c);
        }else{
            buffer[i] = '_';
        }
    }
    puts("[status] Generating header.");
    i=get_value_from_name(T_STRING, "tileset_header", tokens, out);
    fprintf(fp_map, "#ifndef %s_H\n#define %s_H\n\n#include <%s>\n"
        "#include <microfx/ext/gametools.h>\n\nconst unsigned char _%s[] = {",
        buffer, buffer, tokens[i].content, tokens[a].content);
    amount = 0;
    for(i=0;i<(int)size;i++){
        if(data[i] != '\n'){
            found = FALSE;
            for(a=0;a<len;a++){
                if(tiles[a] == data[i]){
                    fprintf(fp_map, "%d, ", a);
                    found = TRUE;
                    amount++;
                }
            }
            if(!found){
                printf("[mapconv] Tile number of '%c' not found\n", data[i]);
                return 5;
            }
        }
    }
    if(amount!=w*h){
        printf("[warning] %d tiles written instead of %d*%d=%d\n", amount, w, h,
            w*h);
    }
    fseek(fp_map, -2L, SEEK_CUR);
    fputs("};", fp_map);
    a=get_value_from_name(T_STRING, "mapname", tokens, out);
    i=get_value_from_name(T_STRING, "tileset_name", tokens, out);
    tw=get_value_from_name(T_INT, "tilewidth", tokens, out);
    th=get_value_from_name(T_INT, "tileheight", tokens, out);
    pw=get_value_from_name(T_INT, "playerwidth", tokens, out);
    ph=get_value_from_name(T_INT, "playerheight", tokens, out);
    padx=get_value_from_name(T_INT, "drawingzone_x", tokens, out);
    pady=get_value_from_name(T_INT, "drawingzone_y", tokens, out);
    dw=get_value_from_name(T_INT, "drawingzone_w", tokens, out);
    dh=get_value_from_name(T_INT, "drawingzone_h", tokens, out);
    fprintf(fp_map, "\n\nMMap %s = {(unsigned char *)_%s, (unsigned char **)"
        "%s, %d, %d, %s, %s, 0, 0, %s, %s, 0, 0, %s, %s, %s, %s};\n\n#endif\n",
        tokens[a].content, tokens[a].content, tokens[i].content, w, h,
        tokens[tw].content, tokens[th].content, tokens[pw].content,
        tokens[ph].content, tokens[padx].content, tokens[pady].content,
        tokens[dw].content, tokens[dh].content);
    /* TODO end */
    fclose(fp_map);
    free(data);
    puts("[status] Generating done! You can now use the generated header "
    "file.");
    return 0;
}
